/**
  ******************************************************************************
  * plik    diskio_stm32f2xx.c 
  * funkcja	nisko poziomowe procedury dla FatFs	do obsugi karty MMC/SD 
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    24-02-2013
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "diskio_stm32f2xx.h"
#include "Definicje_sprzet.h"
#include "RTC_Zegar.h"
#include "Procedury Ogolne.h"
#include "diskio_stm32f2xx_SPI.h"
#include "TIMER_procedury.h"

const uint32_t LINIA_DISC_IO_CLK[LINIE_DISC_IOn]	  ={
	LINIA_DISC_IO_SW_CARD_INSERT_WE_CLK, LINIA_DISC_IO_SW_CARD_PROTECT_WE_CLK, LINIA_DISC_IO_CARD_CS_WY_CLK,
	LINIA_DISC_IO_CARD_SPI_SCK_WY_CLK, LINIA_DISC_IO_CARD_SPI_MOSI_WY_CLK, LINIA_DISC_IO_CARD_SPI_MISO_WE_CLK};

const uint16_t LINIA_DISC_IO_PIN[LINIE_DISC_IOn]	  ={
	LINIA_DISC_IO_SW_CARD_INSERT_WE_PIN, LINIA_DISC_IO_SW_CARD_PROTECT_WE_PIN, LINIA_DISC_IO_CARD_CS_WY_PIN,
	LINIA_DISC_IO_CARD_SPI_SCK_WY_PIN, LINIA_DISC_IO_CARD_SPI_MOSI_WY_PIN, LINIA_DISC_IO_CARD_SPI_MISO_WE_PIN};

GPIO_TypeDef*  LINIA_DISC_IO_PORT[LINIE_DISC_IOn]  	  ={
	LINIA_DISC_IO_SW_CARD_INSERT_WE_PORT, LINIA_DISC_IO_SW_CARD_PROTECT_WE_PORT, LINIA_DISC_IO_CARD_CS_WY_PORT,
	LINIA_DISC_IO_CARD_SPI_SCK_WY_PORT, LINIA_DISC_IO_CARD_SPI_MOSI_WY_PORT, LINIA_DISC_IO_CARD_SPI_MISO_WE_PORT};


static DSTATUS Stat = STA_NOINIT;	/* Disk status */
static BYTE CardType;			/* Card type flags */
static const DWORD socket_state_mask_cp = (1 << 13);//detekcja karty w gniedzie
static const DWORD socket_state_mask_wp = (1 << 7);//detekcja pozycji przecznika zabezpieczenia przed zapisem


/* Definitions for MMC/SDC command */
#define CMD0	(0x40+0)	/* GO_IDLE_STATE */
#define CMD1	(0x40+1)	/* SEND_OP_COND (MMC) */
#define ACMD41	(0xC0+41)	/* SEND_OP_COND (SDC) */
#define CMD8	(0x40+8)	/* SEND_IF_COND */
#define CMD9	(0x40+9)	/* SEND_CSD */
#define CMD10	(0x40+10)	/* SEND_CID */
#define CMD12	(0x40+12)	/* STOP_TRANSMISSION */
#define ACMD13	(0xC0+13)	/* SD_STATUS (SDC) */
#define CMD16	(0x40+16)	/* SET_BLOCKLEN */
#define CMD17	(0x40+17)	/* READ_SINGLE_BLOCK */
#define CMD18	(0x40+18)	/* READ_MULTIPLE_BLOCK */
#define CMD23	(0x40+23)	/* SET_BLOCK_COUNT (MMC) */
#define ACMD23	(0xC0+23)	/* SET_WR_BLK_ERASE_COUNT (SDC) */
#define CMD24	(0x40+24)	/* WRITE_BLOCK */
#define CMD25	(0x40+25)	/* WRITE_MULTIPLE_BLOCK */
#define CMD55	(0x40+55)	/* APP_CMD */
#define CMD58	(0x40+58)	/* READ_OCR */

#if (_MAX_SS != 512) || (_FS_READONLY == 0) || (STM32_SD_DISK_IOCTRL_FORCE == 1)
#define STM32_SD_DISK_IOCTRL   1
#else
#define STM32_SD_DISK_IOCTRL   0
#endif

#define SOCKET_WP_CONNECTED     0//1 /* write-protect socket-switch */
#define SOCKET_CP_CONNECTED     0//1 /* card-present socket-switch */


//---------------------------------------
//procedura odczytu czasu z RTC dla FatFs
DWORD get_fattime (void)
{
#if (ZEGAR_RTC ==ZAMONTOWANY_ZEGAR_RTC)
uint32_t fat_data_czas;

	Odczyt_Czasu_Daty_dla_FAT(&fat_data_czas);
	return fat_data_czas;
#else
	return 0;
#endif
}
//-------------------------------------------------------------------
//pod procedura wczenia zasilania karty (w tej wersji nie istnieje)
void card_power(BYTE on)
{
	on=on;
}
//---------------------------------------------------------------
//procedura kontroli zasilania karty (w tej wersji nie wystpuje)
//Socket power state: 0=off, 1=on
#if (STM32_SD_DISK_IOCTRL == 1)
int chk_power(void)		
{
	return 1;
}
#endif

#if SOCKET_CP_CONNECTED
/* Socket's Card-Present Pin: high = socket empty, low = card inserted */
//------------------------------------------------------------------------------
//procedura inicjacji linii obsugujcej detekcj obecnoci karty SD w gniedzie
//Socket's Card-Present Pin: high = socket empty, low = card inserted 
void socket_cp_init(void)
{
	Linie_DiscIO_Inicjacja(LINIA_DISC_IO_SW_CARD_INSERT_WE,	//linia wejciowa, podciganie do H
						     GPIO_Mode_IN, GPIO_PuPd_UP,  GPIO_OType_PP);
}
//---------------------------------------------------------------
//procedura odczytu stanu czujnika obecnoci karty SD w gniedzie
DWORD socket_is_empty(void)
{
	return ( GPIO_ReadInputData(LINIA_DISC_IO_SW_CARD_INSERT_WE_PORT) & LINIA_DISC_IO_SW_CARD_INSERT_WE_PIN )
																					 ? socket_state_mask_cp : 0;
}
#else

static void socket_cp_init(void)
{
	return;
}

static DWORD socket_is_empty(void)
{
	return 0; /* fake inserted */
}

#endif /* SOCKET_CP_CONNECTED */

#if SOCKET_WP_CONNECTED
/* Socket's Write-Protection Pin: high = write-protected, low = writable */
//--------------------------------------------------------------------------------------------
//procedura inicjacji linii obsugujcej detekcj pozycji przecznika zabezpieczenia karty SD
//Socket's Write-Protection Pin: high = write-protected, low = writable
void socket_wp_init(void)
{
	Linie_DiscIO_Inicjacja(LINIA_DISC_IO_SW_CARD_PROTECT_WE,	//linia wejciowa, podciganie do H
						     GPIO_Mode_IN, GPIO_PuPd_UP,  GPIO_OType_PP);
}
//-----------------------------------------------------------------------------
//procedura odczytu stanu czujnika pozycji przecznika zabezpieczenia karty SD
DWORD socket_is_write_protected(void)
{
	return ( GPIO_ReadInputData(LINIA_DISC_IO_SW_CARD_PROTECT_WE_PORT) & LINIA_DISC_IO_SW_CARD_PROTECT_WE_PIN ) 
																						? socket_state_mask_wp : 0;
}
#else

static void socket_wp_init(void)
{
	return;
}

static DWORD socket_is_write_protected(void)
{
	return 0; /* fake not protected */
}

#endif /* SOCKET_WP_CONNECTED */

//----------------------------------------------------------------
//procedura inicjacji linii obsugujcej wyprowadzenie CS karty SD
void card_CS_init(void)
{
	Linie_DiscIO_Inicjacja(LINIA_DISC_IO_CARD_CS_WY,	//linia wyjciowa, podciganie do H
						      GPIO_Mode_OUT, GPIO_PuPd_UP,  GPIO_OType_PP);
}
//-------------------------
//deselekcja karty SD CS=1
void Deselect_CS_Card(void)
{
	LINIA_DISC_IO_PORT[LINIA_DISC_IO_CARD_CS_WY]->BSRRL = LINIA_DISC_IO_PIN[LINIA_DISC_IO_CARD_CS_WY];
}
//----------------------
//selekcja karty SD CS=0
void Select_CS_Card(void)
{
	LINIA_DISC_IO_PORT[LINIA_DISC_IO_CARD_CS_WY]->BSRRH = LINIA_DISC_IO_PIN[LINIA_DISC_IO_CARD_CS_WY];
}
//-----------------------------------------------------
//pod procedura inicjacji portw interfejsu do komunikacji z kart SD
void power_on (void)
{
	card_power(1);
	socket_cp_init();
	socket_wp_init();
	card_CS_init();
	Delay_ms(250);	//Wait for 250ms
	Deselect_CS_Card();
	SPI_Config();
}
//---------------------------------------------------
//procedura wywoujca odczyt 1 bajtu interfejsem SPI
BYTE rcvr_spi (void)
{
	return stm32_spi_rw(0xff);
}
/* Alternative macro to receive data fast */
#define rcvr_spi_m(dst)  *(dst)=stm32_spi_rw(0xff)

//--------------------------------------------------
//procedura wywoujca zapis 1 bajtu interfejsem SPI
void xmit_spi(BYTE dat)  
{
	stm32_spi_rw(dat);
}
//---------------------------------------
//transmisja pakietu komendy do karty MMC
BYTE send_cmd (
	BYTE cmd,		/* Command byte */
	DWORD arg		/* Argument */
)
{
BYTE n, res;


	if (cmd & 0x80) 	/* ACMD<n> is the command sequence of CMD55-CMD<n> */
	{
		cmd &= 0x7F;
		res = send_cmd(CMD55, 0);
		if (res > 1) return res;
	}

	/* Select the card and wait for ready */
#if (OBSLUGA_KARTY_SD == KARTA_SD_SPI_MODE)
	Deselect_CS_Card();
	Select_CS_Card();
#endif
	if (wait_ready() != 0xFF) 
	{
		return 0xFF;
	}

	/* Send command packet */
	xmit_spi(cmd);						/* Start + Command index */
	xmit_spi((BYTE)(arg >> 24));		/* Argument[31..24] */
	xmit_spi((BYTE)(arg >> 16));		/* Argument[23..16] */
	xmit_spi((BYTE)(arg >> 8));			/* Argument[15..8] */
	xmit_spi((BYTE)arg);				/* Argument[7..0] */
	n = 0x01;							/* Dummy CRC + Stop */
	if (cmd == CMD0) n = 0x95;			/* Valid CRC for CMD0(0) */
	if (cmd == CMD8) n = 0x87;			/* Valid CRC for CMD8(0x1AA) */
	xmit_spi(n);

	/* Receive command response */
	if (cmd == CMD12) rcvr_spi();		/* Skip a stuff byte when stop reading */

	n = 10;								/* Wait for a valid response in timeout of 10 attempts */
	do
		res = rcvr_spi();
	while ((res & 0x80) && --n);

	return res;			/* Return with the response value */
}
//--------------------------------
//oczekiwanie na gotowo karty SD
BYTE wait_ready (void)
{
BYTE res;
extern char flaga_stopera2;


	Start_Stoper2(500);	/* Wait for ready in timeout of 500ms */
	rcvr_spi();
	do
	{
		res = rcvr_spi();
	}
	while ((res != 0xFF) && (flaga_stopera2 ==FALSE));

	return res;
}
//----------------------------------------------
//deselekcja SD Card i zwolnienie magistrali SPI
void release_spi (void)
{
	Deselect_CS_Card();
	rcvr_spi();
}
//-------------------------------
//zwolnienie linii interfejsu SPI
void power_off (void)
{

	if (!(Stat & STA_NOINIT)) 
	{
		Select_CS_Card();
		wait_ready();
		release_spi();
	}

	SPI_Zwolnienie_Linii_Interfejsu();

	card_power(0);

	Stat |= STA_NOINIT;		/* Set STA_NOINIT */
}
//--------------------------------
//odczyt pakietu danych z karty SD
BYTE rcvr_datablock (
	BYTE *buff,			/* Data buffer to store received data */
	UINT btr			/* Byte count (must be multiple of 4) */
)
{
BYTE token;
extern char flaga_stopera1;


	Start_Stoper1(100);
	do 							/* Wait for data packet in timeout of 100ms */
	{
		token = rcvr_spi();
	}
	while ((token == 0xFF) && (flaga_stopera1 ==FALSE));
	if(token != 0xFE) return FALSE;	/* If not valid data token, return with error */

#ifdef STM32_SD_USE_DMA
	stm32_dma_transfer( TRUE, buff, btr );
#else
	do {							/* Receive the data block into buffer */
		rcvr_spi_m(buff++);
		rcvr_spi_m(buff++);
		rcvr_spi_m(buff++);
		rcvr_spi_m(buff++);
	} while (btr -= 4);
#endif /* STM32_SD_USE_DMA */

	rcvr_spi();						/* Discard CRC */
	rcvr_spi();

	return TRUE;					/* Return with success */
}
//-------------------------------------
//transmisja pakietu danych do karty SD
BYTE xmit_datablock (
	const BYTE *buff,	/* 512 byte data block to be transmitted */
	BYTE token			/* Data/Stop token */
)
{
BYTE resp;
#ifndef STM32_SD_USE_DMA
	BYTE wc;
#endif

	if (wait_ready() != 0xFF) return FALSE;

	xmit_spi(token);					/* transmit data token */
	if (token != 0xFD) {	/* Is data token */

#ifdef STM32_SD_USE_DMA
		stm32_dma_transfer( FALSE, buff, 512 );
#else
		wc = 0;
		do {							/* transmit the 512 byte data block to MMC */
			xmit_spi(*buff++);
			xmit_spi(*buff++);
		} while (--wc);
#endif /* STM32_SD_USE_DMA */

		xmit_spi(0xFF);					/* CRC (Dummy) */
		xmit_spi(0xFF);
		resp = rcvr_spi();				/* Receive data response */
		if ((resp & 0x1F) != 0x05)		/* If not accepted, return with error */
			return FALSE;
	}

	return TRUE;
}


//------------------------------------------------------------------------
//procedura inicjacji interfejsu mikrokontrolera do komunikacji z kart SD 
//oraz inicjacja samej karty
DSTATUS stm32f2xx_disk_initialize(BYTE drv)
{
BYTE n, cmd, ty, ocr[4];
extern char flaga_stopera1;

    if (drv) return STA_NOINIT;		/* Supports only single drive */

	power_on();							/* Force socket power on and initialize interface */
	interface_speed(INTERFACE_SLOW);

	for (n = 10; n; n--) rcvr_spi();	/* 80 dummy clocks */

	ty = 0;
	if (send_cmd(CMD0, 0) == 1) 			/* Enter Idle state */
	{
		Start_Stoper1(1000);				/* Initialization timeout of 1000 milliseconds */
		if (send_cmd(CMD8, 0x1AA) == 1) 	/* SDHC */
		{
			for (n = 0; n < 4; n++) ocr[n] = rcvr_spi();		/* Get trailing return value of R7 response */
			if (ocr[2] == 0x01 && ocr[3] == 0xAA) 				/* The card can work at VDD range of 2.7-3.6V */
			{
				while ((flaga_stopera1 ==FALSE) && send_cmd(ACMD41, 1UL << 30));	/* Wait for leaving idle state (ACMD41 with HCS bit) */
				if ((flaga_stopera1 ==FALSE) && send_cmd(CMD58, 0) == 0) 			/* Check CCS bit in the OCR */
				{
					for (n = 0; n < 4; n++) ocr[n] = rcvr_spi();
					ty = (ocr[0] & 0x40) ? CT_SD2 | CT_BLOCK : CT_SD2;
				}
			}
		} 
		else 
		{							/* SDSC or MMC */
			if (send_cmd(ACMD41, 0) <= 1) 	
			{
				ty = CT_SD1; cmd = ACMD41;	/* SDSC */
			} 
			else 
			{
				ty = CT_MMC; cmd = CMD1;	/* MMC */
			}
			while ((flaga_stopera1 ==FALSE) && send_cmd(cmd, 0));			/* Wait for leaving idle state */
			if ((flaga_stopera1 ==TRUE) || send_cmd(CMD16, 512) != 0)	/* Set R/W block length to 512 */
				ty = 0;
		}
	}
	CardType = ty;
	release_spi();

	if (ty) 			/* Initialization succeeded */
	{
		Stat &= ~STA_NOINIT;		/* Clear STA_NOINIT */
		interface_speed(INTERFACE_FAST);
	} 
	else 			/* Initialization failed */
	{
		power_off();
	}


	return Stat;
}
//-------------------------
//przesyanie statusu dysku
DSTATUS stm32f2xx_disk_status (BYTE drv)
{
 	if (drv) return STA_NOINIT;		/* Supports only single drive */
	return Stat;
}
//-----------------------------------------
//odczyt z karty SD sektora/sektorw danych
DRESULT stm32f2xx_disk_read (
	BYTE drv,			/* Physical drive number (0) */
	BYTE *buff,			/* Pointer to the data buffer to store read data */
	DWORD sector,		/* Start sector number (LBA) */
	BYTE count			/* Sector count (1..255) */
)
{
	if (drv || !count) return RES_PARERR;
	if (Stat & STA_NOINIT) return RES_NOTRDY;

	if (!(CardType & CT_BLOCK)) sector *= 512;	/* Convert to byte address if needed */

	if (count == 1) {	/* Single block read */
		if (send_cmd(CMD17, sector) == 0)	{ /* READ_SINGLE_BLOCK */
			if (rcvr_datablock(buff, 512)) {
				count = 0;
			}
		}
	}
	else {				/* Multiple block read */
		if (send_cmd(CMD18, sector) == 0) {	/* READ_MULTIPLE_BLOCK */
			do {
				if (!rcvr_datablock(buff, 512)) {
					break;
				}
				buff += 512;
			} while (--count);
			send_cmd(CMD12, 0);				/* STOP_TRANSMISSION */
		}
	}
	release_spi();

	return count ? RES_ERROR : RES_OK;
}
//-----------------------------------
//zapis sektora/sektorw na karcie SD
DRESULT stm32f2xx_disk_write (
	BYTE drv,			/* Physical drive number (0) */
	const BYTE *buff,	/* Pointer to the data to be written */
	DWORD sector,		/* Start sector number (LBA) */
	BYTE count			/* Sector count (1..255) */
)
{
	if (drv || !count) return RES_PARERR;
	if (Stat & STA_NOINIT) return RES_NOTRDY;
	if (Stat & STA_PROTECT) return RES_WRPRT;

	if (!(CardType & CT_BLOCK)) sector *= 512;	/* Convert to byte address if needed */

	if (count == 1) {	/* Single block write */
		if ((send_cmd(CMD24, sector) == 0)	/* WRITE_BLOCK */
			&& xmit_datablock(buff, 0xFE))
			count = 0;
	}
	else {				/* Multiple block write */
		if (CardType & CT_SDC) send_cmd(ACMD23, count);
		if (send_cmd(CMD25, sector) == 0) {	/* WRITE_MULTIPLE_BLOCK */
			do {
				if (!xmit_datablock(buff, 0xFC)) break;
				buff += 512;
			} while (--count);
			if (!xmit_datablock(0, 0xFD))	/* STOP_TRAN token */
				count = 1;
		}
	}
	release_spi();

	return count ? RES_ERROR : RES_OK;
}
//------------------------------------------------------------
//kontrola statusu dysku
#if (STM32_SD_DISK_IOCTRL == 1)
DRESULT stm32f2xx_disk_ioctl (
	BYTE drv,		/* Physical drive number (0) */
	BYTE ctrl,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
	DRESULT res;
	BYTE n, csd[16], *ptr = buff;
	WORD csize;

	if (drv) return RES_PARERR;

	res = RES_ERROR;

	if (ctrl == CTRL_POWER) {
		switch (*ptr) {
		case 0:		/* Sub control code == 0 (POWER_OFF) */
			if (chk_power())
				power_off();		/* Power off */
			res = RES_OK;
			break;
		case 1:		/* Sub control code == 1 (POWER_ON) */
			power_on();				/* Power on */
			res = RES_OK;
			break;
		case 2:		/* Sub control code == 2 (POWER_GET) */
			*(ptr+1) = (BYTE)chk_power();
			res = RES_OK;
			break;
		default :
			res = RES_PARERR;
		}
	}
	else {
		if (Stat & STA_NOINIT) return RES_NOTRDY;

		switch (ctrl) {
		case CTRL_SYNC :		/* Make sure that no pending write process */
			Select_CS_Card();
			if (wait_ready() == 0xFF)
				res = RES_OK;
			break;

		case GET_SECTOR_COUNT :	/* Get number of sectors on the disk (DWORD) */
			if ((send_cmd(CMD9, 0) == 0) && rcvr_datablock(csd, 16)) {
				if ((csd[0] >> 6) == 1) {	/* SDC version 2.00 */
					csize = csd[9] + ((WORD)csd[8] << 8) + 1;
					*(DWORD*)buff = (DWORD)csize << 10;
				} else {					/* SDC version 1.XX or MMC*/
					n = (csd[5] & 15) + ((csd[10] & 128) >> 7) + ((csd[9] & 3) << 1) + 2;
					csize = (csd[8] >> 6) + ((WORD)csd[7] << 2) + ((WORD)(csd[6] & 3) << 10) + 1;
					*(DWORD*)buff = (DWORD)csize << (n - 9);
				}
				res = RES_OK;
			}
			break;

		case GET_SECTOR_SIZE :	/* Get R/W sector size (WORD) */
			*(WORD*)buff = 512;
			res = RES_OK;
			break;

		case GET_BLOCK_SIZE :	/* Get erase block size in unit of sector (DWORD) */
			if (CardType & CT_SD2) {	/* SDC version 2.00 */
				if (send_cmd(ACMD13, 0) == 0) {	/* Read SD status */
					rcvr_spi();
					if (rcvr_datablock(csd, 16)) {				/* Read partial block */
						for (n = 64 - 16; n; n--) rcvr_spi();	/* Purge trailing data */
						*(DWORD*)buff = 16UL << (csd[10] >> 4);
						res = RES_OK;
					}
				}
			} else {					/* SDC version 1.XX or MMC */
				if ((send_cmd(CMD9, 0) == 0) && rcvr_datablock(csd, 16)) {	/* Read CSD */
					if (CardType & CT_SD1) {	/* SDC version 1.XX */
						*(DWORD*)buff = (((csd[10] & 63) << 1) + ((WORD)(csd[11] & 128) >> 7) + 1) << ((csd[13] >> 6) - 1);
					} else {					/* MMC */
						*(DWORD*)buff = ((WORD)((csd[10] & 124) >> 2) + 1) * (((csd[11] & 3) << 3) + ((csd[11] & 224) >> 5) + 1);
					}
					res = RES_OK;
				}
			}
			break;

		case MMC_GET_TYPE :		/* Get card type flags (1 byte) */
			*ptr = CardType;
			res = RES_OK;
			break;

		case MMC_GET_CSD :		/* Receive CSD as a data block (16 bytes) */
			if (send_cmd(CMD9, 0) == 0		/* READ_CSD */
				&& rcvr_datablock(ptr, 16))
				res = RES_OK;
			break;

		case MMC_GET_CID :		/* Receive CID as a data block (16 bytes) */
			if (send_cmd(CMD10, 0) == 0		/* READ_CID */
				&& rcvr_datablock(ptr, 16))
				res = RES_OK;
			break;

		case MMC_GET_OCR :		/* Receive OCR as an R3 resp (4 bytes) */
			if (send_cmd(CMD58, 0) == 0) {	/* READ_OCR */
				for (n = 4; n; n--) *ptr++ = rcvr_spi();
				res = RES_OK;
			}
			break;

		case MMC_GET_SDSTAT :	/* Receive SD status as a data block (64 bytes) */
			if (send_cmd(ACMD13, 0) == 0) {	/* SD_STATUS */
				rcvr_spi();
				if (rcvr_datablock(ptr, 64))
					res = RES_OK;
			}
			break;

		default:
			res = RES_PARERR;
		}

		release_spi();
	}

	return res;
}
#endif /* _USE_IOCTL != 0 */
//--------------------------------------------------------
//wywoywana co 10ms funkcja kontroli stanu karty w zczu
/* This function must be called in period of 10ms                        */

void disk_timerproc (void)
{
static DWORD pv;
DWORD ns;
BYTE s;


	ns = pv;
	pv = socket_is_empty() | socket_is_write_protected();	/* Sample socket switch */

	if (ns == pv) {                         /* Have contacts stabled? */
		s = Stat;

		if (pv & socket_state_mask_wp)      /* WP is H (write protected) */
			s |= STA_PROTECT;
		else                                /* WP is L (write enabled) */
			s &= ~STA_PROTECT;

		if (pv & socket_state_mask_cp)      /* INS = H (Socket empty) */
			s |= (STA_NODISK | STA_NOINIT);
		else                                /* INS = L (Card inserted) */
			s &= ~STA_NODISK;

		Stat = s;
	}
}


//-------------------------------------------------------------
//procedura inicjacji linii disc_IO interfejsu karty SD
void Linie_DiscIO_Inicjacja(Linie_DiscIO_TypeDef Linia, GPIOMode_TypeDef tryb_pracy, 
														GPIOPuPd_TypeDef  podciaganie,
														GPIOOType_TypeDef typ_linii_wy)
{
   GPIO_InitTypeDef  GPIO_InitStructure;
   
   /* Enable the GPIO_Linie_IO Clock */
   RCC_AHB1PeriphClockCmd(LINIA_DISC_IO_CLK[Linia], ENABLE);

   /* Configure the GPIO_Linie_IO pin*/
   GPIO_InitStructure.GPIO_Pin = LINIA_DISC_IO_PIN[Linia];
   GPIO_InitStructure.GPIO_Mode =tryb_pracy;
   GPIO_InitStructure.GPIO_OType = typ_linii_wy;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
   GPIO_InitStructure.GPIO_PuPd =  podciaganie;
 
   GPIO_Init(LINIA_DISC_IO_PORT[Linia], &GPIO_InitStructure);	
}
