/**
  ******************************************************************************
  * plik    GPIO_procedury.c
  * funkcja	procedury obsugi sprztowych linii IO mikrokontrolera
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    8-08-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  

#include "stm32f2xx_conf.h"
#include "Definicje_sprzet.h"
#include "GPIO_procedury.h"

const uint32_t GPIO_LINIA_IO_CLK[LINIE_IOn]	  ={
	LINIA_LED_ZIELONY_WY_CLK, LINIA_LED_CZERWONY_WY_CLK, LINIA_MCO2_ZEGAR_WY_CLK,
	LINIA_EEP_SDA_CLK, LINIA_EEP_SCL_CLK, LINIA_WYKONAJ_ZDJECIE_CLK};

const uint16_t GPIO_LINIA_IO_PIN[LINIE_IOn]	  ={
	LINIA_LED_ZIELONY_WY_PIN, LINIA_LED_CZERWONY_WY_PIN, LINIA_MCO2_ZEGAR_WY_PIN,
	LINIA_EEP_SDA_PIN, LINIA_EEP_SCL_PIN, LINIA_WYKONAJ_ZDJECIE_PIN};

GPIO_TypeDef*  GPIO_LINIA_IO_PORT[LINIE_IOn]  ={
	LINIA_LED_ZIELONY_WY_PORT, LINIA_LED_CZERWONY_WY_PORT, LINIA_MCO2_ZEGAR_WY_PORT,
	LINIA_EEP_SDA_PORT, LINIA_EEP_SCL_PORT, LINIA_WYKONAJ_ZDJECIE_PORT};

const uint8_t EXTI_LINIA_IO_PIN_SOURC[LINIE_IOn] ={
    LINIA_LED_ZIELONY_WY_PIN_SOURC, LINIA_LED_CZERWONY_WY_PIN_SOURC, LINIA_MCO2_ZEGAR_WY_PIN_SOURC,
	LINIA_EEP_SDA_PIN_SOURC, LINIA_EEP_SCL_PIN_SOURC, LINIA_WYKONAJ_ZDJECIE_PIN_SOURC};
	

//--------------------------------
//procedury inicjacji linii portw
void GPIO_Inicjacja(void)
{
GPIO_InitTypeDef GPIO_InitStructure;

  /* Configure all unused GPIO port pins in Analog Input mode (floating input
     trigger OFF), this will reduce the power consumption and increase the device
     immunity against EMI/EMC *************************************************/

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_Init(GPIOC, &GPIO_InitStructure);
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB |
                         RCC_AHB1Periph_GPIOC | RCC_AHB1Periph_GPIOD |
                         RCC_AHB1Periph_GPIOE, DISABLE); 

  //GPIO_Linie_IO_Inicjacja(Linie_IO_TypeDef Linia, GPIOMode_TypeDef tryb_pracy)
  GPIO_Linie_IO_Inicjacja(LINIA_LED_ZIELONY_WY, GPIO_Mode_OUT, GPIO_OType_PP);
  GPIO_Linie_IO_Inicjacja(LINIA_LED_CZERWONY_WY, GPIO_Mode_OUT, GPIO_OType_PP);
  GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
  GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);
  //Linie EEPROM Inicjacja
  GPIO_Linie_IO_Inicjacja(LINIA_EEP_SDA, GPIO_Mode_OUT, GPIO_OType_OD);
  GPIO_Linie_IO_Inicjacja(LINIA_EEP_SCL, GPIO_Mode_OUT, GPIO_OType_OD);
  GPIO_Linie_IO_High(LINIA_EEP_SDA);
  GPIO_Linie_IO_High(LINIA_EEP_SCL);
  //LINIA_WYKONAJ_ZDJECIE Inicjacja
  GPIO_Linie_IO_Inicjacja(LINIA_WYKONAJ_ZDJECIE, GPIO_Mode_IN, GPIO_OType_PP); 

  GPIO_Linie_IO_Inicjacja(LINIA_MCO2_ZEGAR_WY,  GPIO_Mode_AF, GPIO_OType_PP);

}
//-------------------------------------------------------------
//procedura inicjacji linii IO
void GPIO_Linie_IO_Inicjacja(Linie_IO_TypeDef Linia, GPIOMode_TypeDef tryb_pracy, GPIOOType_TypeDef typ_linii_wy)
{
   GPIO_InitTypeDef  GPIO_InitStructure;
   
   /* Enable the GPIO_Linie_IO Clock */
   RCC_AHB1PeriphClockCmd(GPIO_LINIA_IO_CLK[Linia], ENABLE);

   /* Configure the GPIO_Linie_IO pin*/
   GPIO_InitStructure.GPIO_Pin = GPIO_LINIA_IO_PIN[Linia];
   GPIO_InitStructure.GPIO_Mode =tryb_pracy;
   GPIO_InitStructure.GPIO_OType = typ_linii_wy;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
   if (tryb_pracy ==GPIO_Mode_IN)
   { 
      GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP; 
   }
   else
   {
   	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
   }
 
   GPIO_Init(GPIO_LINIA_IO_PORT[Linia], &GPIO_InitStructure);	
}
//---------------------------------------
//ustawienie stanu wysokiego na linii IO
void GPIO_Linie_IO_High(Linie_IO_TypeDef Linia)
{
//	GPIO_LINIA_IO_PORT[Linia]->BSRR =GPIO_LINIA_IO_PIN[Linia];
	GPIO_LINIA_IO_PORT[Linia]->BSRRL = GPIO_LINIA_IO_PIN[Linia];
}
//---------------------------------------
//ustawienie stanu niskiego na linii IO
void GPIO_Linie_IO_Low(Linie_IO_TypeDef Linia)
{
//	GPIO_LINIA_IO_PORT[Linia]->BRR =GPIO_LINIA_IO_PIN[Linia];
	GPIO_LINIA_IO_PORT[Linia]->BSRRH = GPIO_LINIA_IO_PIN[Linia];
}
//----------------------------------------
//procedura odczytu stanu linii IO
uint32_t GPIO_Linie_IO_Odczyt(Linie_IO_TypeDef Linia)
{
  	return GPIO_ReadInputDataBit(GPIO_LINIA_IO_PORT[Linia], GPIO_LINIA_IO_PIN[Linia]);
}
