/**
  ******************************************************************************
  * plik    Kamera_DMA.c
  * funkcja	Plik procedur zwizanych z transferem DMA danych kamery
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    19-08-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */ 
#include "Kamera_DMA.h"
#include "stm32f2xx_conf.h"   
#include "Definicje_sprzet.h"
#include "Kamera.h"
#include "GPIO_procedury.h"
#include "Procedury Ogolne.h"
#include "TIMER_procedury.h"

/* Private define ------------------------------------------------------------*/
#define DCMI_DR_ADDRESS       0x50050028


DMA_InitTypeDef DMA_InitStructure;
DCMI_InitTypeDef DCMI_InitStructure;

uint8_t	  dcmi_flag_frameri_bak, dcmi_flag_lineri_bak, dcmi_flag_errri_bak, dcmi_flag_ovfri_bak, dcmi_flag_vsyncri_bak;

extern int obraz_parametry_ozo_tab[][3];

//---------------------------------------------
//inicjacja DMA	dla danych kamery MT9D111 -> RAM
void Inicjacja_DMA_MT9D111_do_RAM(uint8_t *bufor_RAM_pliku_obrazka_format, int image_width, int image_height, char bajty_na_piksel)
{
//wersja inicjacji DMA dla wewntrznej pamici RAM 
//dane z wyjcia kamery MT9D111 zapisywane s kanaem DMA do buforu w wntrznej pamici RAM
//rozmiar buforu pojedynczego przesania =ilo bajtw danych linii obrazu * ilo linii <=0xFFFF

uint16_t  ile_bajtow_jednej_linii, calkowita_ilosc_linii_obrazka;
int rozmiar_danych_raw;
uint16_t  rozmiar_przeslania_DMA;


  	ile_bajtow_jednej_linii =image_width * bajty_na_piksel;
	calkowita_ilosc_linii_obrazka =image_height;

  	rozmiar_danych_raw =ile_bajtow_jednej_linii * calkowita_ilosc_linii_obrazka;
  	//cae dane przesane jednym przesaniem DMA
  	rozmiar_przeslania_DMA =rozmiar_danych_raw;

  	/*** Configures the DCMI to interface with the MT9D111 camera module ***/
  	/* Enable DCMI clock */
  	RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);

  	/* DCMI configuration */ 
  	DCMI_DeInit();

	DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_SnapShot;//DCMI_CaptureMode_Continuous;
	DCMI_InitStructure.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;
	DCMI_InitStructure.DCMI_PCKPolarity =/*DCMI_PCKPolarity_Falling;*/DCMI_PCKPolarity_Rising;
	DCMI_InitStructure.DCMI_VSPolarity = DCMI_VSPolarity_Low;//DCMI_VSPolarity_High;
	DCMI_InitStructure.DCMI_HSPolarity = DCMI_HSPolarity_Low;//DCMI_HSPolarity_High;//?!Digh
	DCMI_InitStructure.DCMI_CaptureRate =DCMI_CaptureRate_All_Frame;
	DCMI_InitStructure.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b;

  	//mask interrupt for DCMI
  	DCMI_ITConfig(DCMI_IT_VSYNC, ENABLE);
  	DCMI_ITConfig(DCMI_IT_LINE, ENABLE);
 	DCMI_ITConfig(DCMI_IT_FRAME, ENABLE);
  	DCMI_ITConfig(DCMI_IT_OVF, ENABLE);
  	DCMI_ITConfig(DCMI_IT_ERR, ENABLE);

  	/* Configures the DMA2 to transfer Data from DCMI */
  	/* Enable DMA2 clock */
  	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

  	/* DMA2 Stream1 Configuration */
  	DMA_DeInit(DMA2_Stream1);

  	DMA_InitStructure.DMA_Channel = DMA_Channel_1;  
  	DMA_InitStructure.DMA_PeripheralBaseAddr = DCMI_DR_ADDRESS;	
  	DMA_InitStructure.DMA_Memory0BaseAddr = /*FSMC_LCD_ADDRESS;*/(uint32_t)bufor_RAM_pliku_obrazka_format;
 	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
 	DMA_InitStructure.DMA_BufferSize = rozmiar_przeslania_DMA;//ROZMIAR_BUF_OBRAZU;//1;
  	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;//DMA_MemoryInc_Disable;
  	DMA_InitStructure.DMA_PeripheralDataSize = /*DMA_PeripheralDataSize_Byte;*/DMA_PeripheralDataSize_Word;
  	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;//DMA_MemoryDataSize_HalfWord;
  	DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;//DMA_Mode_Circular;
  	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  	DMA_InitStructure.DMA_FIFOMode = /*DMA_FIFOMode_Enable;*/DMA_FIFOMode_Disable;
  	DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
  	DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  	DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;

  	/* DCMI configuration */
  	DCMI_Init(&DCMI_InitStructure);
	DCMI_JPEGCmd  ( DISABLE );

  	/* DMA2 IRQ channel Configuration */
  	DMA_Init(DMA2_Stream1, &DMA_InitStructure);

  	DMA_ClearITPendingBit ( DMA2_Stream1, 	 DMA_IT_TCIF1  |
                                             DMA_IT_HTIF1  |
                                             DMA_IT_TEIF1  |
                                             DMA_IT_DMEIF1 |
                                             DMA_IT_FEIF1  );
 

  	/* Enable DMA2 stream 1 and DCMI interface then start image capture */
  	DMA_Cmd(DMA2_Stream1, ENABLE); 
  	while ( DISABLE == DMA_GetCmdStatus ( DMA2_Stream1 ) );
  	DCMI_Cmd(ENABLE); 
  	/* Insert 100ms delay: wait 100ms */
  	Delay_ms(200); 
}
//-------------------------------------------------------------------
//procedura przesania danych obrazu z kamery MT9D111 i zapis do wewntrznego buforu w pamici RAM
//dla QQVGA pojedyncze przesanie danych obrazu
void Start_DMA_MT9D111_do_RAM(int image_width, int image_height, char bajty_na_piksel)
{
uint32_t rozmiar_danych_raw_formatu_obrazu;
uint16_t licznik_DMA, piksele_linii, licznik_linii;
char flaga;


	GPIO_Linie_IO_Low(LINIA_LED_ZIELONY_WY);
	GPIO_Linie_IO_Low(LINIA_LED_CZERWONY_WY);

	rozmiar_danych_raw_formatu_obrazu =(image_width *bajty_na_piksel) *image_height;
	if (rozmiar_danych_raw_formatu_obrazu <=0x10000) image_height =1;//pojedyncze przesanie DMA wszystkich linii

	licznik_DMA =DMA_GetCurrDataCounter  (DMA2_Stream1); 
	DCMI_ClearITPendingBit(DCMI_IT_FRAME | DCMI_IT_OVF | DCMI_IT_ERR | DCMI_IT_VSYNC | DCMI_IT_LINE);
	DCMI_ClearFlag (DCMI_FLAG_FRAMERI | DCMI_FLAG_OVFRI | DCMI_FLAG_ERRRI | DCMI_FLAG_VSYNCRI | DCMI_FLAG_LINERI);

	dcmi_flag_frameri_bak =FALSE;
	dcmi_flag_lineri_bak =FALSE;
	dcmi_flag_errri_bak =FALSE;
	dcmi_flag_ovfri_bak =FALSE;

	flaga =FALSE;
	licznik_linii =0;

	DCMI_CaptureCmd(ENABLE);
	
    while ( dcmi_flag_frameri_bak ==FALSE )
	{
		if (dcmi_flag_errri_bak ==TRUE || dcmi_flag_ovfri_bak ==TRUE) 
		{
			break;
		}
		if (dcmi_flag_lineri_bak ==TRUE)
		{
			licznik_linii++;
			if (flaga ==FALSE)
			{
				piksele_linii =DMA_GetCurrDataCounter  (DMA2_Stream1); 
				if (piksele_linii ==licznik_DMA) continue;
				flaga =TRUE;
			}
		}
	}
   
    DCMI_CaptureCmd ( DISABLE );   
    DCMI_Cmd ( DISABLE );
	licznik_DMA =DMA_GetCurrDataCounter  (DMA2_Stream1); 

	GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
	GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);

}
