/**
  ******************************************************************************
  * plik    Katalogi_SD.c
  * funkcja	Plik procedur zwizanych z obsug katalogw i plikw karty SD
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    25-02-2013
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "Katalogi_SD.h"
#include "Definicje_program.h"
#include "Stale_Zmienne_Scenariusze.h"
#include <string.h>

static FATFS fatfs;			/* File system object */
static DIR dir;				/* Directory object */
static FIL fsrc;      			/* file objects */
FILINFO fno;			/* File information object */
FRESULT res;         	/* FatFs function common result code */

const char nazwy_kat_systemowych_tab[][8]={"SY1_PON","SY2_WTO","SY3_SRO","SY4_CZW","SY5_PTK","SY6_SOB","SY7_NIE"};

//-----------------------------------------------------
//procedura weryfikacji systemowych katalogw
void Weryfikacja_Katalogow_Systemowych(uint8_t zadanie)
{
uint8_t x;
char katalog_sys[10];
char ilosc_katalogow_systemowych;

  	f_mount(0, &fatfs);		/* Register volume work area (never fails) */

	if (zadanie ==KATALOGI_WERYFIKACJA_OBECNOSCI)
	{
		ilosc_katalogow_systemowych =7;

		for (x=0; x<ilosc_katalogow_systemowych; x++)
		{
			snprintf(&katalog_sys[0], sizeof(katalog_sys), "/%s", &nazwy_kat_systemowych_tab[x][0]);
		  	res = f_opendir(&dir, &katalog_sys[0]);
		  	if(res) //bd otwarcia katalogu
			{
					//jeli katalog nie istnieje, prba jego zaoenia
					//if (x ==0) res =f_mkfs(0,0,0);
					res = f_mkdir(&katalog_sys[0]);
					if (res !=FR_OK) break;//nie udao si zaoy katalogu systemowego
			}

		}
	}
	f_mount(0, NULL);		/* UNregister volume work area (never fails) */
}
//--------------------------------------------------
//zapis pliku zdjcia do katalogu dnia
void Zapis_Zdjecia_do_Katalogu_Dnia(char *p_sciezka_do_pliku)
{
uint32_t ile_zapisano, ile_do_zapisu, calkowita_ilosc_danych_zapisanych;
extern struct Date_s s_DateStructVar;
extern struct OZO_STRUCT ozo_struct;

	ile_do_zapisu =ozo_struct.rozmiar_pliku_ozo;
	if (ile_do_zapisu >0xFFFF) ile_do_zapisu =0xFFFF;
	ile_zapisano =SD_Card_dane_RW(SD_AKCJA_OTWORZ_DO_ZAPISU, (uint8_t *)(ozo_struct.p_ozo_buf_plik_obraz),
	                              ile_do_zapisu, p_sciezka_do_pliku, 0);

	SD_Card_dane_RW(SD_AKCJA_ZAMKNIJ, NULL, 0, NULL, 0);
	if (ile_zapisano !=ile_do_zapisu)
	{
		//bd
		return;
	} 
	//Delay_ms(20);
	calkowita_ilosc_danych_zapisanych =ile_zapisano;
	while (calkowita_ilosc_danych_zapisanych <ozo_struct.rozmiar_pliku_ozo)
	{
		ile_do_zapisu =ozo_struct.rozmiar_pliku_ozo -calkowita_ilosc_danych_zapisanych;
		if (ile_do_zapisu >0xFFFF) ile_do_zapisu =0xFFFF; 
		ile_zapisano =SD_Card_dane_RW(SD_AKCJA_OTWORZ_I_DOPISZ, (uint8_t *)(ozo_struct.p_ozo_buf_plik_obraz+calkowita_ilosc_danych_zapisanych),
	                              ile_do_zapisu, p_sciezka_do_pliku, 0);

		SD_Card_dane_RW(SD_AKCJA_ZAMKNIJ, NULL, 0, NULL, 0);
		if (ile_zapisano !=ile_do_zapisu) return;//bd
		calkowita_ilosc_danych_zapisanych =calkowita_ilosc_danych_zapisanych +ile_zapisano;
		//Delay_ms(20);
	}
}
//-------------------------------------------------------------------------------
//procedura zwraca ciek aktualnego katalogu dnia z nazw pliku i rozszerzeniem
uint32_t Aktualny_katalog_dnia_Sciezka_Rozszerzenie(char *p_sciezka_do_pliku, char *p_rozszerzenie_pliku)
//we:	*p_sciezka_do_pliku -wskanik do buforu dla stringu ze ciek dostpu dla tworzonego pliku
//		*p_rozszerzenie_pliku -wskanik do buforu ze stringirm rozszerzenia typu tworzonego pliku
//wy:	skompresowana w formacie FAT data-czas utworzenia pliku
{
//char status, rozmiar;
uint16_t numer_pliku;
char nazwa_pliku[9];
signed char numer_dnia_tygodnia/*, typ_katalogu*/;
uint32_t tmp_uint; 

	//wywoanie funkcji aktualizuje struktury biecym czasem i dat
	Odczyt_Czasu_Daty_dla_FAT(&tmp_uint);

	numer_pliku =RTC_ReadBackupRegister(BKP_REJESTR_NUMER_PLIKU);
	if (numer_pliku>ILOSC_ZDJEC_DZIENNIE_MAX) numer_pliku =1;
	snprintf(&nazwa_pliku[0], 9, "ARS_%04d", numer_pliku);
	numer_dnia_tygodnia =s_DateStructVar.Week;
	if (numer_dnia_tygodnia <(TYDZIEN_WARTOSC_MIN) || numer_dnia_tygodnia >(TYDZIEN_WARTOSC_MAX))
	{
		numer_dnia_tygodnia =TYDZIEN_WARTOSC_MIN;
	}
	numer_dnia_tygodnia--;	
//	if (strcmp(p_rozszerzenie_pliku, "RAW") ==0) typ_katalogu =0;	//katalog plikw RAW jest tymczasowym katalogiem dnia
//	else typ_katalogu =0;											//tymczasowy katalog dnia
	snprintf(p_sciezka_do_pliku, ROZMIAR_SCIEZKI_DOSTEPU_MAX, "/%s/%s.%s", &nazwy_kat_systemowych_tab[numer_dnia_tygodnia/*+typ_katalogu*/][0],
																			 &nazwa_pliku[0], p_rozszerzenie_pliku);
	return tmp_uint;
}

//-----------------------------------------------------
//procedura:
//odczyt do buforu danych w iloci rozmiar_buforu
//przesunicie przed odczytem od pocztku pliku o ilo przesuniecie i odczyt do buforu danych w iloci rozmiar_buforu
//zakoczenie i zamknicie operacji na karcie SD
//zapis na karcie SD w iloci rozmiar_buforu
//przesunicie przed zapisem od pocztku pliku o ilo przesuniecie i zapis na SD w iloci rozmiar_buforu
uint32_t SD_Card_dane_RW(uint8_t sd_akcja, uint8_t *p_bufor_pliku, uint32_t rozmiar_buforu, char *p_lista_sciezka, 
                                                                                              uint32_t przesuniecie)
{
//static FATFS fatfs;			/* File system object */
//static FIL fsrc;      			/* file objects */
//FRESULT res;         	/* FatFs function common result code */
uint32_t ile_odczytano, ile_zapisano;

	if (sd_akcja ==SD_AKCJA_OTWORZ_DO_ODCZYTU)
	{
		//inicjacja do odczytu i odczyt
		f_mount(0, &fatfs);		/* Register volume work area (never fails) */
		res = f_open(&fsrc, p_lista_sciezka, FA_OPEN_EXISTING | FA_READ);
		if (res ==0)
		{
			res = f_read(&fsrc,p_bufor_pliku, rozmiar_buforu, &ile_odczytano);
		}
		if (res !=0) ile_odczytano =0;
		return ile_odczytano; 
	}
	if (sd_akcja ==SD_AKCJA_OTWORZ_PRZESUN_CZYTAJ)
	{
		f_mount(0, &fatfs);		/* Register volume work area (never fails) */
		res = f_open(&fsrc, p_lista_sciezka, FA_OPEN_EXISTING | FA_READ);
		if (res ==0)
		{
			res = f_lseek(&fsrc, przesuniecie);
			res = f_read(&fsrc,p_bufor_pliku, rozmiar_buforu, &ile_odczytano);//odczyt kolejnych danych pliku
		}		
		//w przypadku bdu odczytu licznik odczytanych bajtw zawsze =0;
		if (res !=0) ile_odczytano =0; 
		return ile_odczytano;
	}
	if (sd_akcja ==SD_AKCJA_ZAMKNIJ)
	{
		f_close(&fsrc);			/* Close open files */
		f_mount(0, NULL);		/* UNregister volume work area (never fails) */
		return 0;
	}
	if (sd_akcja ==SD_AKCJA_OTWORZ_DO_ZAPISU)
	{
		//inicjacja do zapisu i zapis
		f_mount(0, &fatfs);		/* Register volume work area (never fails) */
		res = f_open(&fsrc, p_lista_sciezka, FA_CREATE_NEW | FA_WRITE);
		if (res ==0)
		{
			res = f_write(&fsrc,p_bufor_pliku, rozmiar_buforu, &ile_zapisano);
		}
		if (res !=0) ile_zapisano =0;
		return ile_zapisano; 
	}
	if (sd_akcja ==SD_AKCJA_OTWORZ_I_DOPISZ)
	{
		//inicjacja do zapisu i dopisania do istniejcego pliku
		Odczyt_Statusu_Pliku_z_SD(p_lista_sciezka);
		f_mount(0, &fatfs);		/* Register volume work area (never fails) */
		res = f_open(&fsrc, p_lista_sciezka, FA_OPEN_EXISTING | FA_WRITE);
		res = f_lseek(&fsrc, fsrc.fsize);
		if (res ==0)
		{
			res = f_write(&fsrc,p_bufor_pliku, rozmiar_buforu, &ile_zapisano);
		}
		if (res !=0) ile_zapisano =0;
		return ile_zapisano; 
	}
	return 0;
}
//-------------------------------------------------------------
//odczyt statusu pliku z SD
char Odczyt_Statusu_Pliku_z_SD(char *p_sciezka_do_pliku)
//we:	p_sciezka_do_pliku -wskanik do cieki dostpu do pliku
//wy:	status: BRAK_BLEDU lub kod bdu
{
char status;

	f_mount(0, &fatfs);		/* Register volume work area (never fails) */
	res=f_stat(p_sciezka_do_pliku, &fno);
	f_mount(0, NULL);		/* UNregister volume work area (never fails) */

	if (res !=FR_OK)
	{
		if (res ==FR_NO_FILE || res ==FR_NO_PATH || res ==FR_INVALID_NAME) status =BLAD_PLIK_NIE_ISTNIEJE;
		else status =BLAD_ODCZYTU_Z_SD;
		return status; //BD
	}

	return BRAK_BLEDU;
}
//---------------------------------------------------
//funkcja wyliczeniowego odczytu kolejnych pozycji katalogu
bool Wyliczanie_kolejnych_pozycji_katalogu(char *p_sciezka_do_pliku, char *p_nazwa_pozycji, int16_t p_pozycja_wpisu_w_katalogu)
{
int16_t x;
char *fn;
int data, czas, rozmiar;


	f_mount(0, &fatfs);		/* Register volume work area (never fails) */
	res = f_opendir(&dir, p_sciezka_do_pliku);
	if (res == FR_OK)
	{
		for (x=0; x<p_pozycja_wpisu_w_katalogu+1;x++)
		{
			res = f_readdir(&dir, &fno);
			if (res != FR_OK || fno.fname[0] == 0) break;

		}
		fn = fno.fname;
		data =fno.fdate;
		czas =fno.ftime;
		rozmiar =fno.fsize;
		if (fno.fattrib & AM_DIR)
		{
			//kolejn odczytan pozycj jest katalog
			snprintf(p_nazwa_pozycji, 30, "%s/", fn); 
		}
		else
		{
			//kolejn odczytan pozycj jest plik
			data =data &0xFFFF;
			czas =czas &0xFFFF;
		    snprintf(p_nazwa_pozycji, 40, "%s d:%05d t:%05d s:%d", fn, data, czas, rozmiar); 
		}
		f_close(&fsrc);			/* Close open files */
		f_mount(0, NULL);		/* UNregister volume work area (never fails) */
		if (res != FR_OK) return FALSE;//jaki bd
		if (fno.fname[0] == 0)
		{
			//nie ma ju pozycji do odczytania
			return FALSE;
		}
		else
		{
			//odczytana kolejna pozycja katalogu
			return TRUE;
		}
	}
	else return FALSE;
}
//--------------------------------------------------------------
//procedura obsugujca polecenie C? odczyt iloci plikw i ich cznego rozmiaru, ilo  podkatalogw w katalogu
FRESULT Polecenie_C_pyt_ilosc_i_rozmiar_plikow_ilosc_podkat_w_kat(char *p_sciezka, int *p_ilosc_rozmiar_plikow_ilosc_podkat)
{
int ile_podkatalogow, ile_plikow, suma_rozmiaru_plikow;

	ile_podkatalogow=0;
	ile_plikow =0;
	suma_rozmiaru_plikow =0;

	f_mount(0, &fatfs);		/* Register volume work area (never fails) */
    res = f_opendir(&dir, p_sciezka);
    if (res != FR_OK) return res;
	while(1)
	{
		res = f_readdir(&dir, &fno);
		if (res != FR_OK || fno.fname[0] == 0) break; //nie ma dalszych plikw do odczytu w katalogu
		if (fno.fname[0] == '.') continue;
		if (fno.fattrib & AM_ARC)
		{
			ile_plikow++;
			suma_rozmiaru_plikow =suma_rozmiaru_plikow +fno.fsize;	
		}
		if (fno.fattrib & AM_DIR)
		{
			ile_podkatalogow++;
		}

	}
	*p_ilosc_rozmiar_plikow_ilosc_podkat =ile_plikow;
	*(p_ilosc_rozmiar_plikow_ilosc_podkat+1) =suma_rozmiaru_plikow;
	*(p_ilosc_rozmiar_plikow_ilosc_podkat+2) =ile_podkatalogow;
	return FR_OK;
}
