/**
  ******************************************************************************
  * plik    main.c
  * funkcja gwna procedura MAIN
  * autor   ARIES Ryszard Szymaniak www.arskam.com
  * wersja  2.15-5-5
  * data    29-11-2013
  * opis    gwna procedura demo dla kodera JPG
  *         sterownik ARS3-MT9D111 obsugujcy modu kamery MT9D111 firmy APTINA
  * ukad:  STM32F217VGT  konfiguracja STM322xG_EVAL
  * przed kompilacj naley w pliku Definicje_sprzet.h zdefiniowa TYP_URZADZENIA
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32f2xx_conf.h"
#include "GPIO_procedury.h"
#include "Procedury Ogolne.h"
#include "Definicje_program.h"
#include "TIMER_procedury.h"
#include "Kamera.h"
#include "Kamera_DMA.h"
#include "RTC_Zegar.h"
#include "Katalogi_SD.h"
#include "Stale_Zmienne_Scenariusze.h"
#include "MT9D111.h"

char format_obrazu;

extern struct OZO_STRUCT ozo_struct;
extern int obraz_ozo_parametry_tab[][4];


int main(void)
{
char status;
uint16_t numer_pliku;
extern int16_t pozycja_wpisu_w_katalogu;
char buf_tmp[ROZMIAR_SCIEZKI_DOSTEPU_MAX+1];
static char bufor_JPG[10000];
extern uint8_t bufor_RAM_pliku_obrazka_format[ROZMIAR_BUF_PLIKU_FORMAT_OBRAZ];
char *p_kodowanie;
//procedura kodera JPG
extern char* encode_image (char typ_kodowania_jpg ,char *output_ptr, int buf_output_rozmiar,
							char quality_factor, int image_width, int image_height, char *p_sciezka_do_pliku);


  /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f10x_xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f10x.c file
     */     

  /* Add your application code here
     */
	GPIO_Inicjacja();
	GPIO_Linie_IO_Low(LINIA_LED_ZIELONY_WY);
	TIMERY_Inicjacja();
	NVIC_Configuration();

 	//modu kamery inicjacja formatu	
  	ozo_struct.ozo_format =NUMER_FORMATU_OZO_QQVGA_BMP;
	ozo_struct.ozo_obraz_szer_pix =obraz_ozo_parametry_tab[ozo_struct.ozo_format][OBRAZ_PARAM_PIKSELE_POZ];
    ozo_struct.ozo_obraz_wys_lin =obraz_ozo_parametry_tab[ozo_struct.ozo_format][OBRAZ_PARAM_LINIE_POZ];
    ozo_struct.ozo_obraz_bajty_na_pix =obraz_ozo_parametry_tab[ozo_struct.ozo_format][OBRAZ_PARAM_BAJTY_NA_PIKS];
	ozo_struct.ozo_format_bak =NUMER_FORMATU_OBRAZU_ZADEN;

	Kamera_Inicjacja(ozo_struct.ozo_obraz_szer_pix, ozo_struct.ozo_obraz_wys_lin, ozo_struct.ozo_obraz_bajty_na_pix);
	status =Kamera_Zmiana_Formatu_Obrazu(ozo_struct.ozo_format);


#if (ZEGAR_RTC ==ZAMONTOWANY_ZEGAR_RTC)
  //inicjacja dziaania zegara RTC
  RTC_Inicjacja();
#endif//ZEGAR_RTC

#if (OBSLUGA_KARTY_SD ==KARTA_SD_SPI_MODE)
	//weryfikacja katalogw systemowych
	Weryfikacja_Katalogow_Systemowych(KATALOGI_WERYFIKACJA_OBECNOSCI);
	//zerowanie parametrw filtru czasowego
	pozycja_wpisu_w_katalogu =0;
#endif//OBSLUGA_KARTY_SD

	if (status ==FALSE)
	{
		GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);
		Delay_ms(300);
		GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
		GPIO_Linie_IO_Low(LINIA_LED_CZERWONY_WY);
	    Delay_ms(300);
		GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);
		GPIO_Linie_IO_Low(LINIA_LED_ZIELONY_WY);
		Delay_ms(300);
		GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
		GPIO_Linie_IO_Low(LINIA_LED_CZERWONY_WY);
		Delay_ms(300);
		GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);
		GPIO_Linie_IO_Low(LINIA_LED_ZIELONY_WY);
	    Delay_ms(300);
	}
    else
	{

		GPIO_Linie_IO_Low(LINIA_LED_ZIELONY_WY);
		GPIO_Linie_IO_Low(LINIA_LED_CZERWONY_WY);
	    Delay_ms(300);
		GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
		GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);
		Delay_ms(300);
		GPIO_Linie_IO_Low(LINIA_LED_ZIELONY_WY);
		GPIO_Linie_IO_Low(LINIA_LED_CZERWONY_WY);
	    Delay_ms(300);
		GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
		GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);
	
    }
  while (1)
  {
	if (GPIO_Linie_IO_Odczyt(LINIA_WYKONAJ_ZDJECIE) ==0) 
	{
		//wykonanie zdjcia w formacie QQVGA 160 x 120 pikseli
		//utworzenie pliku JPG i zapis do katalogu na karcie SD
		ozo_struct.ozo_plik_obraz_typ =OBRAZ_PLIK_JPG;//konwersja do formatu JPG 
  		ozo_struct.ozo_format =NUMER_FORMATU_OZO_QQVGA_BMP;//pozycja w tabeli zawierajcej parametry obrazka: wysoko, szeroko, bajty/piksel
		ozo_struct.ozo_obraz_szer_pix =obraz_ozo_parametry_tab[ozo_struct.ozo_format][OBRAZ_PARAM_PIKSELE_POZ];//160 pikseli
    	ozo_struct.ozo_obraz_wys_lin =obraz_ozo_parametry_tab[ozo_struct.ozo_format][OBRAZ_PARAM_LINIE_POZ];//120 linii
    	ozo_struct.ozo_obraz_bajty_na_pix =obraz_ozo_parametry_tab[ozo_struct.ozo_format][OBRAZ_PARAM_BAJTY_NA_PIKS];//2 bajty/1 piksel obrazu

		//nazwa pliku zdjcia zostanie uzupeniona biecym numerem
		numer_pliku =RTC_ReadBackupRegister(BKP_REJESTR_NUMER_PLIKU);
		numer_pliku++;
		if (numer_pliku>ILOSC_ZDJEC_DZIENNIE_MAX) numer_pliku =1;
		RTC_WriteBackupRegister(BKP_REJESTR_NUMER_PLIKU, numer_pliku);
		//ustawienie rejestrw wymiarw obrazka zgodnie z numerem formatu standardowego wymiaru
		mt9d111_Zmiana_Standardowych_Rozmiarow_Obrazu_Kontekst_B(ozo_struct.ozo_format);
		//obraz w lustrzanym odbiciu pionowym i poziomym ze wzgldu na sposb podczenia moduu
		mt9d111_ustawienie_luster_obrazu(0x03);
	    //rozpoczcie pobierania obrazka z kamery kodowanie i zapis

		Aktualny_katalog_dnia_Sciezka_Rozszerzenie(&buf_tmp[0], "JPG");
		Inicjacja_DMA_MT9D111_do_RAM((uint8_t *)&bufor_RAM_pliku_obrazka_format[0],
								 ozo_struct.ozo_obraz_szer_pix, 
								 ozo_struct.ozo_obraz_wys_lin, 
								 ozo_struct.ozo_obraz_bajty_na_pix); 
		mt9d111_zmian_formatu_danych_obrazu_kontekstu(KONTEKST_B);
		mt9d111_zapis_do_buf_ramki_RGB();
		//konwersja danych surowych obrazka do postaci pliku JPG
		ozo_struct.p_ozo_buf_plik_obraz =&bufor_JPG[0];
			
		p_kodowanie =encode_image (KODOWANIE_JPG_Z_INT_RAM ,//wersja kodera zwizana z lokalizacj kodowanego pliku 
							ozo_struct.p_ozo_buf_plik_obraz, //bufor wyjciowy dla pliku JPG
							10000,				 			//rozmiar buforu JPG
							50, 							 //wspczynnik kompresji typowo 50
							ozo_struct.ozo_obraz_szer_pix,	 //szeroko kodowanego obrazu
							ozo_struct.ozo_obraz_wys_lin,	 //wysoko kodowanego obrazu
							NULL);				 			//nie uywane
															 //wy: wskanik przesunicia od pocztku buforu do obliczenia dugoci
	
	
	
	    ozo_struct.rozmiar_pliku_ozo =(p_kodowanie - ozo_struct.p_ozo_buf_plik_obraz);//wynikowy rozmiar pliku JPG
	    Zapis_Zdjecia_do_Katalogu_Dnia(&buf_tmp[0]);

		Delay_ms(100);
	}
  }
}

/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  */  
//PUTCHAR_PROTOTYPE
//{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the USART */
//  USART_SendData(EVAL_COM1, (uint8_t) ch);

  /* Loop until the end of transmission */
//  while (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_TC) == RESET)
//  {}

//  return ch;
//}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */


/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
