/**
  ******************************************************************************
  * @file    Project/STM32F2xx_StdPeriph_Template/stm32f2xx_it.c 
  * @author  MCD Application Team
  * @version V1.1.0
  * @date    13-April-2012
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and 
  *          peripherals interrupt service routine.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f2xx_it.h"
#include "Definicje_sprzet.h"
#include "TIMER_procedury.h"
#include "diskio_stm32f2xx.h"

/** @addtogroup Template_Project
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

//TIMER4 stopery
extern char flaga_stopera1;
extern char flaga_stopera2;
extern char flaga_stopera3;
extern char flaga_stopera4;
extern char flaga_stopera5;


/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief   This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
   disk_timerproc();
}

/******************************************************************************/
/*                 STM32F2xx Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f2xx.s).                                               */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
/*void PPP_IRQHandler(void)
{
}*/

/**
  * @}
  */ 
  /**
  * port szeregowy USART1
  * wykorzystywany do komunikacji z PC
  */
//----------------------------------------------------
void USART_1_IRQHandler(void)	
{ 
	//obsuga przerwania USART1
  if(USART_GetITStatus(USART_1, USART_IT_RXNE) != RESET)
  {
  }
  if(USART_GetITStatus(USART_1, USART_IT_TXE) != RESET)
  {   
  }
}

/**
  * @}
  */ 
  /**
  * port szeregowy USART2
  * 
  */
//----------------------------------------------------
void USART_2_IRQHandler(void)	
{ 
	//obsuga przerwania USART2
  if(USART_GetITStatus(USART_2, USART_IT_RXNE) != RESET)
  {
  }
  if(USART_GetITStatus(USART_2, USART_IT_TXE) != RESET)
  {   
  }
}
//--------------------------------------------------
void TIM2_IRQHandler(void)
{
  if (TIM_GetITStatus(TIM2, TIM_IT_CC3) != RESET)
  {
    TIM_ClearITPendingBit(TIM2, TIM_IT_CC3);
  }
}

//--------------------------------------------------
//TIM4 stopery 1-4
//obsuga 
void TIM4_IRQHandler(void)
{
  if (TIM_GetITStatus(TIM4, TIM_IT_CC1) != RESET)
  {
    //kana CC1 steruje odliczaniem czasu Stopera1
    TIM_ClearITPendingBit(TIM4, TIM_IT_CC1);
	//wyczenie przerwania kanau
	TIM_ITConfig(TIM4, TIM_IT_CC1, DISABLE);
	flaga_stopera1 =TRUE;
  }	
  if (TIM_GetITStatus(TIM4, TIM_IT_CC2) != RESET)
  {
    //kana CC2 steruje odliczaniem czasu Stopera2
    TIM_ClearITPendingBit(TIM4, TIM_IT_CC2);
	//wyczenie przerwania kanau
	TIM_ITConfig(TIM4, TIM_IT_CC2, DISABLE);
	flaga_stopera2 =TRUE;
  }
  if (TIM_GetITStatus(TIM4, TIM_IT_CC3) != RESET)
  {
  }	
  if (TIM_GetITStatus(TIM4, TIM_IT_CC4) != RESET)
  {
  }	

}

//--------------------------------------------------
//TIM5 licznik czasu przerwy transmisji UART
void TIM5_IRQHandler(void)
{
  if (TIM_GetITStatus(TIM5, TIM_IT_CC1) != RESET)
  {
  }	
  if (TIM_GetITStatus(TIM5, TIM_IT_CC2) != RESET)
  {
  }	  
}

//-------------------------------------------------
//przerwania DCMI
void DCMI_IRQHandler(void)
{
extern uint8_t	  dcmi_flag_frameri_bak, dcmi_flag_lineri_bak, dcmi_flag_errri_bak, dcmi_flag_ovfri_bak, dcmi_flag_vsyncri_bak;

	if (DCMI_GetFlagStatus(DCMI_FLAG_VSYNCRI) ==SET)
	{
		dcmi_flag_vsyncri_bak =TRUE;
	}
	if (DCMI_GetFlagStatus(DCMI_FLAG_LINERI) ==SET)
	{
	}
	if (DCMI_GetFlagStatus(DCMI_FLAG_LINEMI) ==SET)
	{
		dcmi_flag_lineri_bak =TRUE;
	}
	if (DCMI_GetFlagStatus(DCMI_FLAG_FRAMERI) ==SET)
	{
		dcmi_flag_frameri_bak =TRUE;
	}
	if (DCMI_GetFlagStatus(DCMI_FLAG_ERRRI) ==SET)
	{
		dcmi_flag_errri_bak =TRUE;
	}
	if (DCMI_GetFlagStatus(DCMI_FLAG_OVFRI) ==SET)
	{
		dcmi_flag_ovfri_bak =TRUE;
	}

	DCMI_ClearFlag(DCMI_FLAG_VSYNCRI);
    DCMI_ClearFlag(DCMI_FLAG_LINERI);
	DCMI_ClearFlag(DCMI_FLAG_LINEMI);
	DCMI_ClearFlag(DCMI_FLAG_FRAMERI);
	DCMI_ClearFlag(DCMI_FLAG_ERRRI);
	DCMI_ClearFlag(DCMI_FLAG_OVFRI);
}
//------------------------------------------------------
//przerwanie DMA
void DMA2_Stream1_IRQHandler(void)
{
    static uint8_t ucCount = 0;
    DMA_ClearITPendingBit ( DMA2_Stream1, DMA_IT_TCIF1 );
    ucCount++;
}
/******************************************************************************/
/**
  * @brief  This function handles SDIO global interrupt request.
  * @param  None
  * @retval None
  */
void SDIO_IRQHandler(void)
{
}

/**
  * @brief  This function handles DMA2 Stream3 or DMA2 Stream6 global interrupts
  *         requests.
  * @param  None
  * @retval None
  */
void SD_SDIO_DMA_IRQHANDLER(void)
{
}

//-------------------------------------------------------
//obsuga przerwania linii portu
void EXTI0_IRQHandler(void)
{									   
}
//-------------------------------------------------------
//obsuga przerwania linii portu 
void EXTI1_IRQHandler(void)
{									   
}
//-------------------------------------------------------
//obsuga przerwania linii portu
void EXTI2_IRQHandler(void)
{									   
}
//-------------------------------------------------------
//obsuga przerwania linii portu
void EXTI3_IRQHandler(void)
{									   
}
//-------------------------------------------------------
//obsuga przerwania linii portu
void EXTI9_5_IRQHandler(void)
{									   
}

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
