/**
  ******************************************************************************
  * plik    TIMER_procedury.c
  * funkcja	Plik procedur zwizanych z TIMER-ami
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    23-07-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "TIMER_procedury.h"
#include "stm32f2xx_conf.h"
#include "Definicje_sprzet.h"
#include "Definicje_program.h"

TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;
TIM_ICInitTypeDef  TIM_ICInitStructure;

extern uint16_t CZAS_PRZERWY_TRANSMISJI_UART_GP;//extern uint16_t czas_przerwy_transmisji_UART1;
char flaga_stopera2;	//flaga sygnalizujca zakoczenie pomiaru przez Licznik2
char flaga_stopera1;	//flaga sygnalizujca zakoczenie pomiaru przez Licznik1
char flaga_stopera3;
char flaga_stopera4;
char flaga_stopera5;

//PRZYPORZDKOWANIE STOPERW
//TIMER4
//CC1 Stoper1:
//				FatFs -karta SD timeouty oczekiwania na zakoczenie procedur karty
//CC2 Stoper2:	FatFs -oczekiwanie na gotowo karty SD
//CC3 Stoper3:	
//CC3 Stoper4:	
//TIMER5
//CC1 Stoper5:	
//CC2 

//procedura inicjacji wszystkich Timerw uywanych przez program
void TIMERY_Inicjacja(void)
{
  //System Clocks Configuration inicjacja impulsw zegarowych dla TIMER4
  TIMER4_RCC_Configuration();
  //inicjacja TIMERA4 zwizanego z systemem stoperw i pomiarem odcinkw czasu
  TIMER4_Inicjacja();
  //System Clocks Configuration inicjacja impulsw zegarowych dla TIMER5
  TIMER5_RCC_Configuration();

  SysTick_Ini();

}

 /*-------------------------------------------------
  * @brief  Configures the different system clocks.
  * @param  None
  * @retval None
  */
//podczenie sygnau zegarowego do TIMER2
void TIMER2_RCC_Configuration(void)
{
  /* Setup the microcontroller system. Initialize the Embedded Flash Interface,  
     initialize the PLL and update the SystemFrequency variable. */

  /* PCLK1 = HCLK/4 */
//  RCC_PCLK1Config(RCC_HCLK_Div4);
  /* TIM2 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
}
//---------------------------------------------------
void TIMER2_Inicjacja(void)
{
  /* ---------------------------------------------------------------
    TIM2 Configuration: kana TIM2_CH3 w poczeniu z DMA obsuguje zapis 
	danych z kamery PO6030 do buforu w pamici RAM mikrokontrolera
    TIM2CLK = 36 MHz, Prescaler = 36, TIM2 counter clock = 1MHz(1us)
  --------------------------------------------------------------- */
uint16_t PrescalerValue;

  PrescalerValue = (uint16_t) ((SystemCoreClock / 2) / 1000000) - 1;

  /* TIM2 disable counter */
  TIM_Cmd(TIM2, DISABLE); 

  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = 65535;
  TIM_TimeBaseStructure.TIM_Prescaler = 1;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  /* Prescaler configuration */
  TIM_PrescalerConfig(TIM2, PrescalerValue, TIM_PSCReloadMode_Immediate);

 TIM_ICStructInit(&TIM_ICInitStructure);
  //kana 3 "input capture"
  TIM_ICInitStructure.TIM_Channel =TIM_Channel_3;
  //reakcja na zbocze narastajce sygnau PCLK
 TIM_ICInitStructure.TIM_ICPolarity =/*TIM_ICPolarity_Falling*/TIM_ICPolarity_Rising;
  TIM_ICInitStructure.TIM_ICSelection =TIM_ICSelection_DirectTI;
  //bez preskalera zdarze, kade zdarzenie (zbocze) uwzgldniane
 TIM_ICInitStructure.TIM_ICPrescaler =TIM_ICPSC_DIV1;
  //bez filtra
  TIM_ICInitStructure.TIM_ICFilter =0;

  TIM_ICInit(TIM2, &TIM_ICInitStructure);

  /* TIM IT enable */
  //dla celw testowych wczanie przerwania zliczajcego zbocza PCLK
  //TIM_ITConfig(TIM2, TIM_IT_CC3, ENABLE);

  /* TIM2 enable counter */
  TIM_Cmd(TIM2, ENABLE); 
}

//----------------------------------------
//podczenie sygnau zegarowego do TIMER3
void TIMER3_RCC_Configuration(void)
{
  /* PCLK1 = HCLK/4 */
  RCC_PCLK1Config(RCC_HCLK_Div4);
  /* TIM2 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
}
//---------------------------------------------------
void TIMER3_Inicjacja(void)
{
}

//podczenie sygnau zegarowego do TIMER4
void TIMER4_RCC_Configuration(void)
{
  /* Setup the microcontroller system. Initialize the Embedded Flash Interface,  
     initialize the PLL and update the SystemFrequency variable. */

  /* PCLK1 = HCLK/4 */
  //RCC_PCLK1Config(RCC_HCLK_Div4);
  /* TIM4 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);

}
//---------------------------------------------------
//inicjacja TIMERA4 zwizanego ze stoperami i pomiarem odcinkw czasu
void TIMER4_Inicjacja(void)
{
  /* ---------------------------------------------------------------
    TIM4 Configuration: Output Compare Timing Mode:
    TIM4 counter clock = 1kHz
  --------------------------------------------------------------- */
uint16_t PrescalerValue;

  PrescalerValue = (uint16_t) ((SystemCoreClock / 2) / 1000) - 1;
  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = 65535;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);

  /* Prescaler configuration */
  //TIM_PrescalerConfig(TIM4, 36000, TIM_PSCReloadMode_Immediate); //1.
  TIM_PrescalerConfig(TIM4, PrescalerValue, TIM_PSCReloadMode_Immediate); //2V

  /* Output Compare Timing Mode configuration: Channel1 */
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = ((U_19200_PRZERWA_01MS *2)/10)+1;//wstpne odliczanie przedziau czasowego przerwy transmisji dla 19200
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC2Init(TIM4, &TIM_OCInitStructure);

  TIM_OC2PreloadConfig(TIM4, TIM_OCPreload_Disable);


  /* TIM IT enable */
  TIM_ITConfig(TIM4, TIM_IT_CC2, DISABLE);
  //TIM_ITConfig(TIM4, TIM_IT_CC1, ENABLE);

  /* TIM4 enable counter */
  TIM_Cmd(TIM4, ENABLE); 
}
//----------------------------------------
//podczenie sygnau zegarowego do TIMER5
void TIMER5_RCC_Configuration(void)
{
  /* Setup the microcontroller system. Initialize the Embedded Flash Interface,  
     initialize the PLL and update the SystemFrequency variable. */

  /* PCLK1 = HCLK/4 */
  //RCC_PCLK1Config(RCC_HCLK_Div4);
  /* TIM5 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);
}
//---------------------------------------------------
//inicjacja TIMERA5 
void TIMER5_Inicjacja(void)
{
  /* ---------------------------------------------------------------
    TIM5 Configuration: Output Compare Timing Mode:
    1. TIM5CLK =  MHz, Prescaler = , TIM5 counter clock = 1kHz
  --------------------------------------------------------------- */
uint16_t PrescalerValue;

  PrescalerValue = (uint16_t) ((SystemCoreClock / 2) / 1000) - 1;
  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = 65535;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM5, &TIM_TimeBaseStructure);

  /* Prescaler configuration */
  TIM_PrescalerConfig(TIM5, PrescalerValue, TIM_PSCReloadMode_Immediate);

  /* Output Compare Timing Mode configuration: Channel1 */
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = ((U_19200_PRZERWA_01MS *2)/10)+1;//wstpne odliczanie przedzia-u czasowego przerwy transmisji dla 19200
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC2Init(TIM5, &TIM_OCInitStructure);

  TIM_OC2PreloadConfig(TIM5, TIM_OCPreload_Disable);


  /* TIM IT enable */
  TIM_ITConfig(TIM5, TIM_IT_CC2, DISABLE);
  //TIM_ITConfig(TIM4, TIM_IT_CC1, ENABLE);

  /* TIM5 enable counter */
  TIM_Cmd(TIM5, ENABLE); 

}


//procedury inicjacji timera systemowego
void SysTick_Ini(void)
{
	/* Setup SysTick Timer for 10 millisecond interrupts, also enables Systick and Systick-Interrupt */
	if (SysTick_Config(SystemCoreClock / 100))
	{
		/* Capture error */
		while (1);
	}
}
//------------------------------------------------------------------
//wykorzystany Timer4 i kana CC1
//odliczanie czasu przerwy z rozdzielczoci 1ms STOPERA1
void Start_Stoper1(uint16_t odliczany_czas_ms)
{
uint16_t CCR1_Val;

  /* Output Compare Timing Mode configuration: Channel1 */
  TIM_ITConfig(TIM4, TIM_IT_CC1, DISABLE);

  CCR1_Val=TIM_GetCounter(TIM4);
  CCR1_Val = CCR1_Val + odliczany_czas_ms;
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = CCR1_Val;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC1Init(TIM4, &TIM_OCInitStructure);

  TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Disable);
  /* TIM IT enable */
  flaga_stopera1 =FALSE;
  TIM_ClearITPendingBit(TIM4, TIM_IT_CC1);
  TIM_ITConfig(TIM4, TIM_IT_CC1, ENABLE);
}
//------------------------------------------------------------------
//wykorzystany Timer4 i kana CC1
//zatrzymanie Stopera1
void Stop_Stoper1(void)
{
	TIM_ITConfig(TIM4, TIM_IT_CC1, DISABLE);
	flaga_stopera1 =TRUE;
}
//--------------------------------------
//start Stopera2 odlicza czas z rozdzielczoci 1ms STOPERA2
//wykorzystany Timer4 i kana CC2
void Start_Stoper2(int odliczany_czas_1ms)
{
uint16_t CCR2_Val;

  /* Output Compare Timing Mode configuration: Channel1 */
  TIM_ITConfig(TIM4, TIM_IT_CC2, DISABLE);
  CCR2_Val=TIM_GetCounter(TIM4);
  CCR2_Val = CCR2_Val + odliczany_czas_1ms;
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = CCR2_Val;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC2Init(TIM4, &TIM_OCInitStructure);

  TIM_OC2PreloadConfig(TIM4, TIM_OCPreload_Disable);
  /* TIM IT enable */
  flaga_stopera2 =FALSE;
  TIM_ClearITPendingBit(TIM4, TIM_IT_CC2);
  TIM_ITConfig(TIM4, TIM_IT_CC2, ENABLE);
}
//------------------------------------------------------------------
//wykorzystany Timer4 i kana CC2
//zatrzymanie Stopera2
void Stop_Stoper2(void)
{
	TIM_ITConfig(TIM4, TIM_IT_CC2, DISABLE);
	//flaga_stopera2 =TRUE;
}
//------------------------------------------------------------------
//wykorzystany Timer4 i kana CC3
//odliczanie czasu przerwy z rozdzielczoci 1ms 
void Start_Stoper3(uint16_t odliczany_czas_ms)
{
uint16_t CCR3_Val;

  /* Output Compare Timing Mode configuration: Channel1 */
  TIM_ITConfig(TIM4, TIM_IT_CC3, DISABLE);

  CCR3_Val=TIM_GetCounter(TIM4);
  CCR3_Val = CCR3_Val + odliczany_czas_ms;
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = CCR3_Val;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC3Init(TIM4, &TIM_OCInitStructure);

  TIM_OC3PreloadConfig(TIM4, TIM_OCPreload_Disable);
  TIM_ClearITPendingBit(TIM4, TIM_IT_CC3);
  /* TIM IT enable */
  flaga_stopera3 =FALSE;
  TIM_ClearITPendingBit(TIM4, TIM_IT_CC3);
  TIM_ITConfig(TIM4, TIM_IT_CC3, ENABLE);
}
//------------------------------------------------------------------
//wykorzystany Timer4 i kana CC3
//zatrzymanie Stopera3
void Stop_Stoper3(void)
{
	TIM_ITConfig(TIM4, TIM_IT_CC3, DISABLE);
	//flaga_stopera2 =TRUE;
}
//------------------------------------------------------------------
//------------------------------------------------------------------
//wykorzystany Timer4 i kana CC4
//odliczanie czasu przerwy z rozdzielczoci 1ms 
void Start_Stoper4(uint16_t odliczany_czas_ms)
{
uint16_t CCR4_Val;

  /* Output Compare Timing Mode configuration: Channel1 */
  TIM_ITConfig(TIM4, TIM_IT_CC4, DISABLE);

  CCR4_Val=TIM_GetCounter(TIM4);
  CCR4_Val = CCR4_Val + odliczany_czas_ms;
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = CCR4_Val;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC4Init(TIM4, &TIM_OCInitStructure);

  TIM_OC4PreloadConfig(TIM4, TIM_OCPreload_Disable);
  TIM_ClearITPendingBit(TIM4, TIM_IT_CC4);
  /* TIM IT enable */
  flaga_stopera4 =FALSE;
  TIM_ClearITPendingBit(TIM4, TIM_IT_CC4);
  TIM_ITConfig(TIM4, TIM_IT_CC4, ENABLE);
}
//------------------------------------------------------------------
//wykorzystany Timer4 i kana CC4
//zatrzymanie Stopera4
void Stop_Stoper4(void)
{
	TIM_ITConfig(TIM4, TIM_IT_CC4, DISABLE);
	//flaga_stopera2 =TRUE;
}
//------------------------------------------------------------------
//wykorzystany Timer5 i kana CC1
//odliczanie czasu przerwy z rozdzielczoci 1ms STOPERA5
void Start_Stoper5(uint16_t odliczany_czas_ms)
{
uint16_t CCR1_Val;

  /* Output Compare Timing Mode configuration: Channel1 */
  TIM_ITConfig(TIM5, TIM_IT_CC1, DISABLE);

  CCR1_Val=TIM_GetCounter(TIM5);
  CCR1_Val = CCR1_Val + odliczany_czas_ms;
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = CCR1_Val;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC1Init(TIM5, &TIM_OCInitStructure);

  TIM_OC1PreloadConfig(TIM5, TIM_OCPreload_Disable);
  /* TIM IT enable */
  flaga_stopera5 =FALSE;
  TIM_ClearITPendingBit(TIM5, TIM_IT_CC1);
  TIM_ITConfig(TIM5, TIM_IT_CC1, ENABLE);
}
//------------------------------------------------------------------
//wykorzystany Timer5 i kana CC1
//zatrzymanie Stopera5
void Stop_Stoper5(void)
{
	TIM_ITConfig(TIM5, TIM_IT_CC1, DISABLE);
}

