 /**
  ******************************************************************************
  * plik    RTC_Zegar.c
  * funkcja	Plik procedur zwizanych z obsug zegara czasu rzeczywistego RTC
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    21-02-2013
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  

#include "Definicje_sprzet.h"
#include "stm32f2xx_conf.h"                        // STM32F10x Library Definitions
#include "stm32f2xx_pwr.h"
#include "stm32f2xx_rtc.h"
#include "RTC_Zegar.h"
#include <stdio.h>

struct Date_s s_DateStructVar;//struktura kalendarza
struct Time_s s_TimeStructVar;//struktura zegara\

RTC_TimeTypeDef RTC_TimeStructure;
RTC_DateTypeDef RTC_DateStructure;
RTC_InitTypeDef RTC_InitStructure;
__IO uint32_t AsynchPrediv = 0, SynchPrediv = 0;


//------------------------------------
//inicjacja zegara czasu rzeczywistego
void RTC_Inicjacja(void)
{
  /* Enable PWR and BKP clocks */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

  /* Allow access to BKP Domain and RTC*/
  PWR_BackupAccessCmd(ENABLE);

  /* Reset Backup Domain */
  //BKP_DeInit();
  /* Enable LSE 32.768 kHz external oscillator*/
  /* Enable the LSE OSC */
  RCC_LSEConfig(RCC_LSE_ON);

  /* Wait till LSE is ready */  
  while(RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
  {
  }

  /* Select the RTC Clock Source */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
  
  SynchPrediv = 0xFF;
  AsynchPrediv = 0x7F;

  /* Enable the RTC Clock */
  RCC_RTCCLKCmd(ENABLE);

  /* Wait for RTC APB registers synchronisation */
  RTC_WaitForSynchro();

    /* Configure the RTC data register and RTC prescaler */
    RTC_InitStructure.RTC_AsynchPrediv = AsynchPrediv;
    RTC_InitStructure.RTC_SynchPrediv = SynchPrediv;
    RTC_InitStructure.RTC_HourFormat = RTC_HourFormat_24;
   
    /* Check on RTC init */
    if (RTC_Init(&RTC_InitStructure) == ERROR)
    {
     
    }

/*  Backup SRAM ***************************************************************/
  /* Enable BKPRAM Clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_BKPSRAM, ENABLE);

  /* Enable the Backup SRAM low power Regulator to retain it's content in VBAT mode */
  PWR_BackupRegulatorCmd(ENABLE);

  /* Wait until the Backup SRAM low power Regulator is ready */
  while(PWR_GetFlagStatus(PWR_FLAG_BRR) == RESET)
  {
  }

}
//----------------------------
//odczyt czasu z zegara RTC
void RTC_Odczyt_czasu(char *p_buf)
{

  /* Get the current Time */
  RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
  s_TimeStructVar.HourHigh=(uint8_t)(RTC_TimeStructure.RTC_Hours)/10;
  s_TimeStructVar.HourLow=(uint8_t)(RTC_TimeStructure.RTC_Hours)%10;
  s_TimeStructVar.MinHigh=(uint8_t)(RTC_TimeStructure.RTC_Minutes)/10;
  s_TimeStructVar.MinLow=(uint8_t)(RTC_TimeStructure.RTC_Minutes)%10;
  s_TimeStructVar.SecHigh=(uint8_t)(RTC_TimeStructure.RTC_Seconds)/10;
  s_TimeStructVar.SecLow=(uint8_t)(RTC_TimeStructure.RTC_Seconds)%10;

  //printf("\n\r  The current time is :  %0.2d:%0.2d:%0.2d \n\r", RTC_TimeStructure.RTC_Hours, RTC_TimeStructure.RTC_Minutes, RTC_TimeStructure.RTC_Seconds);
  snprintf(p_buf, 12, "%d%d:%d%d:%d%d\0", s_TimeStructVar.HourHigh, s_TimeStructVar.HourLow,
  										  s_TimeStructVar.MinHigh, s_TimeStructVar.MinLow,
										  s_TimeStructVar.SecHigh, s_TimeStructVar.SecLow);

}
//-----------------------------------------------------------------------------------
//kompresja parametrw czasu, zapis do rejestrw RTC lub powrt z komunikatem 'false'
bool Zegar_kompresja(void)
{
uint32_t wartosc;

	//godziny
	wartosc =s_TimeStructVar.HourHigh;
    wartosc =wartosc *10;
	wartosc =wartosc +s_TimeStructVar.HourLow;
	if (wartosc >GODZINY_WARTOSC_MAX) return FALSE;
	RTC_TimeStructure.RTC_Hours =wartosc;
	//minuty
	wartosc =s_TimeStructVar.MinHigh;
    wartosc =wartosc *10;
	wartosc =wartosc +s_TimeStructVar.MinLow;
	if (wartosc >MINUTY_WARTOSC_MAX) return FALSE;
	RTC_TimeStructure.RTC_Minutes =wartosc;
	//sekundy
	wartosc =s_TimeStructVar.SecHigh;
    wartosc =wartosc *10;
	wartosc =wartosc +s_TimeStructVar.SecLow;
	if (wartosc >SEKUNDY_WARTOSC_MAX) return FALSE;
	RTC_TimeStructure.RTC_Seconds =wartosc;

  	if(RTC_SetTime(RTC_Format_BIN, &RTC_TimeStructure) == ERROR)
  	{
    	return FALSE;
  	} 
 	else
  	{
    	return TRUE;
	}
}
//---------------------------------------
//odczyt i wywietlenie daty z zegara RTC
void RTC_Odczyt_Daty(char *p_buf)
{
int rok;

//  CheckForDaysElapsed();
  Kalendarz_Inicjacja();
  rok =s_DateStructVar.Year +ROK_WARTOSC_MIN;
  snprintf(p_buf, 12, "%0.4d-%0.2d-%0.2d\0", rok, s_DateStructVar.Month, s_DateStructVar.Day);
}
//---------------------
//inicjacja kalendarza
void Kalendarz_Inicjacja(void)
{
	//odczyt z rejestru zegara daty 
    RTC_GetDate(RTC_Format_BIN, &RTC_DateStructure);
    //i zapis do struktury
    s_DateStructVar.Year  =RTC_DateStructure.RTC_Year;		//rok 0-99
	s_DateStructVar.Month =RTC_DateStructure.RTC_Month;		//miesic
    s_DateStructVar.Day	  =RTC_DateStructure.RTC_Date;		//dzie miesica
	s_DateStructVar.Week  =RTC_DateStructure.RTC_WeekDay;	//dzie tygodnia 1-poniedziaek, .. 7-niedziela
}
//--------------------------------------
//kompresja daty i zapis do rejestru zegara daty
void Kalendarz_kompresja(void)
{
	RTC_DateStructure.RTC_Year 		=s_DateStructVar.Year;
	RTC_DateStructure.RTC_Month 	=s_DateStructVar.Month;
	RTC_DateStructure.RTC_Date		=s_DateStructVar.Day;
	RTC_DateStructure.RTC_WeekDay	=s_DateStructVar.Week;

	RTC_SetDate(RTC_Format_BIN, &RTC_DateStructure); 	
} 
//-------------------------------------------------------------------------------
//odczyt czasu i daty w formacie FAT
void Odczyt_Czasu_Daty_dla_FAT(uint32_t *fat_data_czas)
{
uint32_t data_czas, wartosc;
char tmp_tab[13];

	RTC_Odczyt_Daty(&tmp_tab[0]);
	RTC_Odczyt_czasu(&tmp_tab[0]);
	//rok
	data_czas =((s_DateStructVar.Year & KOMPRESJA_KALENDARZ_MASKA_ROKU) <<25);
	//miesic
    data_czas =data_czas + ((s_DateStructVar.Month	& KOMPRESJA_KALENDARZ_MASKA_MIESIAC) <<21); 
	//dzie
	data_czas =data_czas + ((s_DateStructVar.Day &	KOMPRESJA_KALENDARZ_MASKA_DZIEN) <<16);
	//godzina
	wartosc =s_TimeStructVar.HourHigh;
    wartosc =wartosc *10;
	wartosc =wartosc +s_TimeStructVar.HourLow;
	if (wartosc >GODZINY_WARTOSC_MAX) wartosc =GODZINY_WARTOSC_MIN;
	data_czas =data_czas + (wartosc<<11);
	//minuty
	wartosc =s_TimeStructVar.MinHigh;
    wartosc =wartosc *10;
	wartosc =wartosc +s_TimeStructVar.MinLow;
	if (wartosc >MINUTY_WARTOSC_MAX) wartosc =MINUTY_WARTOSC_MIN;
	data_czas =data_czas + (wartosc<<5);
	//sekundy
	wartosc =s_TimeStructVar.SecHigh;
    wartosc =wartosc *10;
	wartosc =wartosc +s_TimeStructVar.SecLow;
	if (wartosc >SEKUNDY_WARTOSC_MAX) wartosc =SEKUNDY_WARTOSC_MIN;
	wartosc =wartosc >>2;
	data_czas =data_czas +wartosc;
	*fat_data_czas =data_czas;
}

