/**
  ******************************************************************************
  * plik    diskio_stm32f2xx.h
  * funkcja	plik nagwkowy nisko poziomowych procedur dla FatFs	
  * 		do obsugi karty MMC/SD 
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    24-02-2013
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __DISKIO_STM32F2XX_H
#define __DISKIO_STM32F2XX_H

#include "diskio.h"
#include "stm32f2xx_conf.h"

//#define STM32_SD_USE_DMA	0

//definicje linii discIO tworzcych interfejs do komunikcji z kart MMC/SD
#define LINIE_DISC_IOn					6
typedef enum
{
	LINIA_DISC_IO_SW_CARD_INSERT_WE		=0,
	LINIA_DISC_IO_SW_CARD_PROTECT_WE	=1,	  
	LINIA_DISC_IO_CARD_CS_WY			=2,
	LINIA_DISC_IO_CARD_SPI_SCK_WY		=3,
    LINIA_DISC_IO_CARD_SPI_MOSI_WY		=4,
	LINIA_DISC_IO_CARD_SPI_MISO_WE		=5
}Linie_DiscIO_TypeDef;

	//LINIA_DISC_IO_SW_CARD_INSERT_WE linia detekcji obecnoci karty SD w gniedzie
#define LINIA_DISC_IO_SW_CARD_INSERT_WE_PORT		GPIOC
#define LINIA_DISC_IO_SW_CARD_INSERT_WE_CLK			RCC_AHB1Periph_GPIOC             
#define LINIA_DISC_IO_SW_CARD_INSERT_WE_PIN			GPIO_Pin_13
#define LINIA_DISC_IO_SW_CARD_INSERT_WE_PIN_SOURC	GPIO_PinSource13
	//LINIA_DISC_IO_SW_CARD_PROTECT_WE linia detekcji zabezpieczenia przed zapisem karty SD
#define LINIA_DISC_IO_SW_CARD_PROTECT_WE_PORT		GPIOD
#define LINIA_DISC_IO_SW_CARD_PROTECT_WE_CLK		RCC_AHB1Periph_GPIOD             
#define LINIA_DISC_IO_SW_CARD_PROTECT_WE_PIN		GPIO_Pin_7
#define LINIA_DISC_IO_SW_CARD_PROTECT_WE_PIN_SOURC	GPIO_PinSource7
	//LINIA_DISC_IO_SW_CARD_CS_WY linia CS karty SD
#define LINIA_DISC_IO_CARD_CS_WY_PORT				GPIOB
#define LINIA_DISC_IO_CARD_CS_WY_CLK				RCC_AHB1Periph_GPIOB             
#define LINIA_DISC_IO_CARD_CS_WY_PIN				GPIO_Pin_12
#define LINIA_DISC_IO_CARD_CS_WY_PIN_SOURC			GPIO_PinSource12
   //LINIA_DISC_IO_SW_CARD_SPI_SCK_WY linia SCK interfejsu SPI karty SD
#define LINIA_DISC_IO_CARD_SPI_SCK_WY_PORT			GPIOB
#define LINIA_DISC_IO_CARD_SPI_SCK_WY_CLK			RCC_AHB1Periph_GPIOB             
#define LINIA_DISC_IO_CARD_SPI_SCK_WY_PIN			GPIO_Pin_13
#define LINIA_DISC_IO_CARD_SPI_SCK_WY_PIN_SOURC		GPIO_PinSource13
	//LINIA_DISC_IO_SW_CARD_SPI_MOSI_WY linia MOSI interfejsu SPI karty SD
#define LINIA_DISC_IO_CARD_SPI_MOSI_WY_PORT			GPIOB
#define LINIA_DISC_IO_CARD_SPI_MOSI_WY_CLK			RCC_AHB1Periph_GPIOB            
#define LINIA_DISC_IO_CARD_SPI_MOSI_WY_PIN			GPIO_Pin_15
#define LINIA_DISC_IO_CARD_SPI_MOSI_WY_PIN_SOURC	GPIO_PinSource15
	//LINIA_DISC_IO_SW_CARD_SPI_MISO_WE linia MISO interfejsu SPI karty SD
#define LINIA_DISC_IO_CARD_SPI_MISO_WE_PORT			GPIOB
#define LINIA_DISC_IO_CARD_SPI_MISO_WE_CLK			RCC_AHB1Periph_GPIOB            
#define LINIA_DISC_IO_CARD_SPI_MISO_WE_PIN			GPIO_Pin_14
#define LINIA_DISC_IOCARD_SPI_MISO_WE_PIN_SOURC		GPIO_PinSource14




DWORD get_fattime (void);
void card_power(BYTE on);
int chk_power(void);
void socket_cp_init(void);
DWORD socket_is_empty(void);
void socket_wp_init(void);
DWORD socket_is_write_protected(void);
void card_CS_init(void);
void Deselect_CS_Card(void);
void Select_CS_Card(void);
void power_on (void);
BYTE rcvr_spi (void);
void xmit_spi(BYTE dat);
BYTE send_cmd (BYTE cmd, DWORD arg);
BYTE wait_ready (void);
void release_spi (void);
void power_off (void);
BYTE rcvr_datablock (BYTE *buff, UINT btr);
BYTE xmit_datablock (const BYTE *buff, BYTE token);

DSTATUS stm32f2xx_disk_initialize(BYTE drv);
DSTATUS stm32f2xx_disk_status (BYTE drv);
DRESULT stm32f2xx_disk_read (BYTE drv, BYTE *buff, DWORD sector, BYTE count);
DRESULT stm32f2xx_disk_write (BYTE drv, const BYTE *buff, DWORD sector, BYTE count);
DRESULT stm32f2xx_disk_ioctl (BYTE drv, BYTE ctrl, void *buff);
void disk_timerproc (void);

void Linie_DiscIO_Inicjacja(Linie_DiscIO_TypeDef Linia, GPIOMode_TypeDef tryb_pracy, 
														GPIOPuPd_TypeDef  podciaganie,
														GPIOOType_TypeDef typ_linii_wy);

#endif
