 /**
  ******************************************************************************
  * plik    RTC_Zegar.c
  * funkcja	Plik nagwkowy procedur zwizanych z obsug zegara czasu rzeczywistego RTC
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    21-02-2013
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "stm32f2xx_conf.h"
#include "Definicje_program.h"

/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __RTC_ZEGAR_H
#define __RTC_ZEGAR_H

/* Date Structure definition */
//definicja struktury kalendarza
struct Date_s
{
  uint8_t Date_Update;
  uint8_t Week;
  uint8_t Month;
  uint8_t Day;
  uint16_t Year;
};
extern struct Date_s s_DateStructVar;
/* Time Structure definition */
//definicja struktury zegara
struct Time_s
{
  uint8_t SecLow;
  uint8_t SecHigh;
  uint8_t MinLow;
  uint8_t MinHigh;
  uint8_t HourLow;
  uint8_t HourHigh;
};
extern struct Time_s s_TimeStructVar;


#define BKP_REJESTR_NUMER_PLIKU				RTC_BKP_DR0
#define BKP_REJESTR_RTC_ALARM_H				RTC_BKP_DR1
#define BKP_REJESTR_RTC_ALARM_L				RTC_BKP_DR2
#define REJESTR_FLAG_POZ_ALARM_CYKLICZNY_AKTYWNY	0
#define BKP_REJESTR_FLAG_STANU				RTC_BKP_DR3

//b.15-9 numer roku, b.8-5 numer miesica, b.4-0 numer dnia
#define KOMPRESJA_KALENDARZ_MASKA_ROKU		0x7F
#define KOMPRESJA_KALENDARZ_MASKA_MIESIAC	0x0F 
#define KOMPRESJA_KALENDARZ_MASKA_DZIEN		0x1F

#define SECONDS_IN_DAY 86399

#define GODZINY_WARTOSC_MIN					0
#define GODZINY_WARTOSC_MAX					23
#define MINUTY_WARTOSC_MIN					0
#define MINUTY_WARTOSC_MAX					59
#define SEKUNDY_WARTOSC_MIN					0
#define SEKUNDY_WARTOSC_MAX					59

#define ROK_WARTOSC_MIN						1980
#define ROK_WARTOSC_MAX						2107
#define MIESIAC_WARTOSC_MIN					1
#define MIESIAC_WARTOSC_MAX					12
#define DZIEN_WARTOSC_MIN					1
#define DZIEN_WARTOSC_MAX					31
#define TYDZIEN_WARTOSC_MIN					1
#define TYDZIEN_WARTOSC_MAX					7

void RTC_Inicjacja(void);
void RTC_Odczyt_czasu(char *p_buf);
bool Zegar_kompresja(void);
void Kalendarz_Inicjacja(void);
void Kalendarz_kompresja(void);
void RTC_Odczyt_Daty(char *p_buf);
void Odczyt_Czasu_Daty_dla_FAT(uint32_t *fat_data_czas);

#endif


